@@ -1,5 +1,5 @@
 // Program     : x10-amh
-// Version     : 1.05
+// Version     : 1.06
 // Description : X10 Model CP-290 control program (available at Radio Shack)
 //               (computer control for X10 modules)
 //               This application allows you to control lights and appliances
@@ -44,7 +44,7 @@
 #define X10_MODE_TOMORROW 2
 #define X10_MODE_CLEAR    0
 
-#define VERSION         "v1.05"
+#define VERSION         "v1.06"
 
 /*
   Write n bytes from file descriptor, and don't return until done
@@ -278,8 +278,37 @@ x10::checksum( u_char *bytes, int len )
 }
 
 /*
-  convert string of numbers in the form of "1,2,3,4" to a bitmap appropriate
-  for the X10 control module
+  convert string of numbers in the form of "1,2,3,4,5,6,7" to a bitmap that
+  represents the day of the week
+*/
+
+static u_short
+strtodm( const char *string )
+{
+#define ITODM(i)                (((i)<=7&&(i)>=1)?(1<<((i)-1)):(0))
+  u_short rval=0, cur=0;
+
+  if( string ) {
+
+    for( int i=0; string[i]; i++ ) {
+
+      if( isdigit( string[i] ) ) cur = cur*10 + string[i] - '0';
+      else {
+
+        rval |= ITODM( cur );
+        cur = 0;
+      }
+    }
+
+    rval |= ITODM( cur );
+  }
+
+  return rval;
+}
+
+/*
+  convert string of numbers in the form of "1,2,3,4" to a bitmap that
+  represents the device to turn on or off
 */
 
 static u_short
@@ -897,7 +926,7 @@ x10::setEvent( int i, char housecode, const char *devmap, const char *daymap,
                int mode, int minute, int hour, int dimlevel, int func )
 {
   events[i][0] = ( mode   & 0x0f );
-  events[i][1] = ( ( strtobm( daymap ? daymap : "1,2,3,4,5,6,7" ) ) >> 9 );
+  events[i][1] = ( strtodm( daymap ? daymap : "1,2,3,4,5,6,7" ) );
   events[i][2] = ( hour   & 0x1f );
   events[i][3] = ( minute & 0x3f );
   events[i][4] = ( strtobm( devmap ? devmap : "1" ) >> 8 ) & 0xff;
